# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging as std_logging
from collections import OrderedDict
import re
from typing import Dict, Callable, Mapping, MutableMapping, MutableSequence, Optional, Sequence, Tuple, Type, Union

from googlecloudsdk.generated_clients.gapic_clients.run_v2 import gapic_version as package_version

from google.api_core.client_options import ClientOptions
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.auth import credentials as ga_credentials   # type: ignore
from google.oauth2 import service_account              # type: ignore
import cloudsdk.google.protobuf


try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from googlecloudsdk.generated_clients.gapic_clients.run_v2.types import k8s_min
from googlecloudsdk.generated_clients.gapic_clients.run_v2.types import source_upload
from .transports.base import SourceUploadTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import SourceUploadGrpcAsyncIOTransport
from .client import SourceUploadClient

try:
    from google.api_core import client_logging  # type: ignore
    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)

class SourceUploadAsyncClient:
    """The Cloud Run Source Upload API allows you to upload source
    code to deploy Cloud Run resources.
    """

    _client: SourceUploadClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = SourceUploadClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = SourceUploadClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = SourceUploadClient._DEFAULT_ENDPOINT_TEMPLATE
    _DEFAULT_UNIVERSE = SourceUploadClient._DEFAULT_UNIVERSE

    crypto_key_path = staticmethod(SourceUploadClient.crypto_key_path)
    parse_crypto_key_path = staticmethod(SourceUploadClient.parse_crypto_key_path)
    service_path = staticmethod(SourceUploadClient.service_path)
    parse_service_path = staticmethod(SourceUploadClient.parse_service_path)
    common_billing_account_path = staticmethod(SourceUploadClient.common_billing_account_path)
    parse_common_billing_account_path = staticmethod(SourceUploadClient.parse_common_billing_account_path)
    common_folder_path = staticmethod(SourceUploadClient.common_folder_path)
    parse_common_folder_path = staticmethod(SourceUploadClient.parse_common_folder_path)
    common_organization_path = staticmethod(SourceUploadClient.common_organization_path)
    parse_common_organization_path = staticmethod(SourceUploadClient.parse_common_organization_path)
    common_project_path = staticmethod(SourceUploadClient.common_project_path)
    parse_common_project_path = staticmethod(SourceUploadClient.parse_common_project_path)
    common_location_path = staticmethod(SourceUploadClient.common_location_path)
    parse_common_location_path = staticmethod(SourceUploadClient.parse_common_location_path)

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SourceUploadAsyncClient: The constructed client.
        """
        return SourceUploadClient.from_service_account_info.__func__(SourceUploadAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SourceUploadAsyncClient: The constructed client.
        """
        return SourceUploadClient.from_service_account_file.__func__(SourceUploadAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(cls, client_options: Optional[ClientOptions] = None):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return SourceUploadClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> SourceUploadTransport:
        """Returns the transport used by the client instance.

        Returns:
            SourceUploadTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = SourceUploadClient.get_transport_class

    def __init__(self, *,
            credentials: Optional[ga_credentials.Credentials] = None,
            transport: Optional[Union[str, SourceUploadTransport, Callable[..., SourceUploadTransport]]] = "grpc_asyncio",
            client_options: Optional[ClientOptions] = None,
            client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
            ) -> None:
        """Instantiates the source upload async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,SourceUploadTransport,Callable[..., SourceUploadTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the SourceUploadTransport constructor.
                If set to None, a transport is chosen automatically.
                NOTE: "rest" transport functionality is currently in a
                beta state (preview). We welcome your feedback via an
                issue in this library's source repository.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = SourceUploadClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,

        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(std_logging.DEBUG):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.cloud.run_v2.SourceUploadAsyncClient`.",
                extra = {
                    "serviceName": "google.cloud.run.v2.SourceUpload",
                    "universeDomain": getattr(self._client._transport._credentials, "universe_domain", ""),
                    "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                    "credentialsInfo": getattr(self.transport._credentials, "get_cred_info", lambda: None)(),
                } if hasattr(self._client._transport, "_credentials") else {
                    "serviceName": "google.cloud.run.v2.SourceUpload",
                    "credentialsType": None,
                }
            )

    async def upload_source(self,
            request: Optional[Union[source_upload.UploadSourceRequest, dict]] = None,
            *,
            parent: Optional[str] = None,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
            ) -> source_upload.UploadSourceResponse:
        r"""Uploads a source archive to a Google Cloud Storage
        bucket through Cloud Run. The uploaded source object
        should be used for Cloud Run resource deployments. User
        is responsible for managing the lifecycle of the
        uploaded object.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from googlecloudsdk.generated_clients.gapic_clients import run_v2

            async def sample_upload_source():
                # Create a client
                client = run_v2.SourceUploadAsyncClient()

                # Initialize request argument(s)
                request = run_v2.UploadSourceRequest(
                    service="service_value",
                    parent="parent_value",
                )

                # Make the request
                response = await client.upload_source(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[googlecloudsdk.generated_clients.gapic_clients.run_v2.types.UploadSourceRequest, dict]]):
                The request object. The request message for the
                UploadSource method.
            parent (:class:`str`):
                Required. The project and location in which the source
                archive should be uploaded to, specified in the format
                ``projects/*/locations/*``.

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            googlecloudsdk.generated_clients.gapic_clients.run_v2.types.UploadSourceResponse:
                The response message for the
                UploadSource method.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent]
        has_flattened_params = len([param for param in flattened_params if param is not None]) > 0
        if request is not None and has_flattened_params:
            raise ValueError("If the `request` argument is set, then none of "
                             "the individual field arguments should be set.")

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, source_upload.UploadSourceRequest):
            request = source_upload.UploadSourceRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[self._client._transport.upload_source]

        header_params = {}

        routing_param_regex = re.compile('^projects/[^/]+/locations/(?P<location>[^/]+)$')
        regex_match = routing_param_regex.match(request.parent)
        if regex_match and regex_match.group("location"):
            header_params["location"] = regex_match.group("location")

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "SourceUploadAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(gapic_version=package_version.__version__)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):   # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = cloudsdk.google.protobuf.__version__


__all__ = (
    "SourceUploadAsyncClient",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
import json  # type: ignore

from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.api_core import exceptions as core_exceptions
from google.api_core import retry as retries
from google.api_core import rest_helpers
from google.api_core import rest_streaming
from google.api_core import gapic_v1
import cloudsdk.google.protobuf

from cloudsdk.google.protobuf import json_format
from google.api_core import operations_v1

from requests import __version__ as requests_version
import dataclasses
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings


from google.iam.v1 import iam_policy_pb2  # type: ignore
from google.iam.v1 import policy_pb2  # type: ignore
from googlecloudsdk.generated_clients.gapic_clients.run_v2.types import instance
from googlecloudsdk.generated_clients.gapic_clients.run_v2.types import instance as gcr_instance
from google.longrunning import operations_pb2  # type: ignore


from .rest_base import _BaseInstancesRestTransport
from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore

try:
    from google.api_core import client_logging  # type: ignore
    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = logging.getLogger(__name__)

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=f"requests@{requests_version}",
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = cloudsdk.google.protobuf.__version__


class InstancesRestInterceptor:
    """Interceptor for Instances.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the InstancesRestTransport.

    .. code-block:: python
        class MyCustomInstancesInterceptor(InstancesRestInterceptor):
            def pre_create_instance(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_create_instance(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_delete_instance(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_delete_instance(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_get_iam_policy(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_iam_policy(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_get_instance(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_instance(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_list_instances(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_list_instances(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_set_iam_policy(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_set_iam_policy(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_start_instance(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_start_instance(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_stop_instance(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_stop_instance(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_test_iam_permissions(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_test_iam_permissions(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_update_instance(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_update_instance(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = InstancesRestTransport(interceptor=MyCustomInstancesInterceptor())
        client = InstancesClient(transport=transport)


    """
    def pre_create_instance(self, request: gcr_instance.CreateInstanceRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[gcr_instance.CreateInstanceRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for create_instance

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_create_instance(self, response: operations_pb2.Operation) -> operations_pb2.Operation:
        """Post-rpc interceptor for create_instance

        DEPRECATED. Please use the `post_create_instance_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_create_instance` interceptor runs
        before the `post_create_instance_with_metadata` interceptor.
        """
        return response

    def post_create_instance_with_metadata(self, response: operations_pb2.Operation, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[operations_pb2.Operation, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for create_instance

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_create_instance_with_metadata`
        interceptor in new development instead of the `post_create_instance` interceptor.
        When both interceptors are used, this `post_create_instance_with_metadata` interceptor runs after the
        `post_create_instance` interceptor. The (possibly modified) response returned by
        `post_create_instance` will be passed to
        `post_create_instance_with_metadata`.
        """
        return response, metadata

    def pre_delete_instance(self, request: instance.DeleteInstanceRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[instance.DeleteInstanceRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for delete_instance

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_delete_instance(self, response: operations_pb2.Operation) -> operations_pb2.Operation:
        """Post-rpc interceptor for delete_instance

        DEPRECATED. Please use the `post_delete_instance_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_delete_instance` interceptor runs
        before the `post_delete_instance_with_metadata` interceptor.
        """
        return response

    def post_delete_instance_with_metadata(self, response: operations_pb2.Operation, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[operations_pb2.Operation, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for delete_instance

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_delete_instance_with_metadata`
        interceptor in new development instead of the `post_delete_instance` interceptor.
        When both interceptors are used, this `post_delete_instance_with_metadata` interceptor runs after the
        `post_delete_instance` interceptor. The (possibly modified) response returned by
        `post_delete_instance` will be passed to
        `post_delete_instance_with_metadata`.
        """
        return response, metadata

    def pre_get_iam_policy(self, request: iam_policy_pb2.GetIamPolicyRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[iam_policy_pb2.GetIamPolicyRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for get_iam_policy

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_get_iam_policy(self, response: policy_pb2.Policy) -> policy_pb2.Policy:
        """Post-rpc interceptor for get_iam_policy

        DEPRECATED. Please use the `post_get_iam_policy_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_get_iam_policy` interceptor runs
        before the `post_get_iam_policy_with_metadata` interceptor.
        """
        return response

    def post_get_iam_policy_with_metadata(self, response: policy_pb2.Policy, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[policy_pb2.Policy, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for get_iam_policy

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_get_iam_policy_with_metadata`
        interceptor in new development instead of the `post_get_iam_policy` interceptor.
        When both interceptors are used, this `post_get_iam_policy_with_metadata` interceptor runs after the
        `post_get_iam_policy` interceptor. The (possibly modified) response returned by
        `post_get_iam_policy` will be passed to
        `post_get_iam_policy_with_metadata`.
        """
        return response, metadata

    def pre_get_instance(self, request: instance.GetInstanceRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[instance.GetInstanceRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for get_instance

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_get_instance(self, response: instance.Instance) -> instance.Instance:
        """Post-rpc interceptor for get_instance

        DEPRECATED. Please use the `post_get_instance_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_get_instance` interceptor runs
        before the `post_get_instance_with_metadata` interceptor.
        """
        return response

    def post_get_instance_with_metadata(self, response: instance.Instance, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[instance.Instance, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for get_instance

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_get_instance_with_metadata`
        interceptor in new development instead of the `post_get_instance` interceptor.
        When both interceptors are used, this `post_get_instance_with_metadata` interceptor runs after the
        `post_get_instance` interceptor. The (possibly modified) response returned by
        `post_get_instance` will be passed to
        `post_get_instance_with_metadata`.
        """
        return response, metadata

    def pre_list_instances(self, request: instance.ListInstancesRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[instance.ListInstancesRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for list_instances

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_list_instances(self, response: instance.ListInstancesResponse) -> instance.ListInstancesResponse:
        """Post-rpc interceptor for list_instances

        DEPRECATED. Please use the `post_list_instances_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_list_instances` interceptor runs
        before the `post_list_instances_with_metadata` interceptor.
        """
        return response

    def post_list_instances_with_metadata(self, response: instance.ListInstancesResponse, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[instance.ListInstancesResponse, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for list_instances

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_list_instances_with_metadata`
        interceptor in new development instead of the `post_list_instances` interceptor.
        When both interceptors are used, this `post_list_instances_with_metadata` interceptor runs after the
        `post_list_instances` interceptor. The (possibly modified) response returned by
        `post_list_instances` will be passed to
        `post_list_instances_with_metadata`.
        """
        return response, metadata

    def pre_set_iam_policy(self, request: iam_policy_pb2.SetIamPolicyRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[iam_policy_pb2.SetIamPolicyRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for set_iam_policy

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_set_iam_policy(self, response: policy_pb2.Policy) -> policy_pb2.Policy:
        """Post-rpc interceptor for set_iam_policy

        DEPRECATED. Please use the `post_set_iam_policy_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_set_iam_policy` interceptor runs
        before the `post_set_iam_policy_with_metadata` interceptor.
        """
        return response

    def post_set_iam_policy_with_metadata(self, response: policy_pb2.Policy, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[policy_pb2.Policy, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for set_iam_policy

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_set_iam_policy_with_metadata`
        interceptor in new development instead of the `post_set_iam_policy` interceptor.
        When both interceptors are used, this `post_set_iam_policy_with_metadata` interceptor runs after the
        `post_set_iam_policy` interceptor. The (possibly modified) response returned by
        `post_set_iam_policy` will be passed to
        `post_set_iam_policy_with_metadata`.
        """
        return response, metadata

    def pre_start_instance(self, request: instance.StartInstanceRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[instance.StartInstanceRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for start_instance

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_start_instance(self, response: operations_pb2.Operation) -> operations_pb2.Operation:
        """Post-rpc interceptor for start_instance

        DEPRECATED. Please use the `post_start_instance_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_start_instance` interceptor runs
        before the `post_start_instance_with_metadata` interceptor.
        """
        return response

    def post_start_instance_with_metadata(self, response: operations_pb2.Operation, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[operations_pb2.Operation, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for start_instance

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_start_instance_with_metadata`
        interceptor in new development instead of the `post_start_instance` interceptor.
        When both interceptors are used, this `post_start_instance_with_metadata` interceptor runs after the
        `post_start_instance` interceptor. The (possibly modified) response returned by
        `post_start_instance` will be passed to
        `post_start_instance_with_metadata`.
        """
        return response, metadata

    def pre_stop_instance(self, request: instance.StopInstanceRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[instance.StopInstanceRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for stop_instance

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_stop_instance(self, response: operations_pb2.Operation) -> operations_pb2.Operation:
        """Post-rpc interceptor for stop_instance

        DEPRECATED. Please use the `post_stop_instance_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_stop_instance` interceptor runs
        before the `post_stop_instance_with_metadata` interceptor.
        """
        return response

    def post_stop_instance_with_metadata(self, response: operations_pb2.Operation, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[operations_pb2.Operation, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for stop_instance

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_stop_instance_with_metadata`
        interceptor in new development instead of the `post_stop_instance` interceptor.
        When both interceptors are used, this `post_stop_instance_with_metadata` interceptor runs after the
        `post_stop_instance` interceptor. The (possibly modified) response returned by
        `post_stop_instance` will be passed to
        `post_stop_instance_with_metadata`.
        """
        return response, metadata

    def pre_test_iam_permissions(self, request: iam_policy_pb2.TestIamPermissionsRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[iam_policy_pb2.TestIamPermissionsRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for test_iam_permissions

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_test_iam_permissions(self, response: iam_policy_pb2.TestIamPermissionsResponse) -> iam_policy_pb2.TestIamPermissionsResponse:
        """Post-rpc interceptor for test_iam_permissions

        DEPRECATED. Please use the `post_test_iam_permissions_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_test_iam_permissions` interceptor runs
        before the `post_test_iam_permissions_with_metadata` interceptor.
        """
        return response

    def post_test_iam_permissions_with_metadata(self, response: iam_policy_pb2.TestIamPermissionsResponse, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[iam_policy_pb2.TestIamPermissionsResponse, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for test_iam_permissions

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_test_iam_permissions_with_metadata`
        interceptor in new development instead of the `post_test_iam_permissions` interceptor.
        When both interceptors are used, this `post_test_iam_permissions_with_metadata` interceptor runs after the
        `post_test_iam_permissions` interceptor. The (possibly modified) response returned by
        `post_test_iam_permissions` will be passed to
        `post_test_iam_permissions_with_metadata`.
        """
        return response, metadata

    def pre_update_instance(self, request: gcr_instance.UpdateInstanceRequest, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[gcr_instance.UpdateInstanceRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for update_instance

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Instances server.
        """
        return request, metadata

    def post_update_instance(self, response: operations_pb2.Operation) -> operations_pb2.Operation:
        """Post-rpc interceptor for update_instance

        DEPRECATED. Please use the `post_update_instance_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the Instances server but before
        it is returned to user code. This `post_update_instance` interceptor runs
        before the `post_update_instance_with_metadata` interceptor.
        """
        return response

    def post_update_instance_with_metadata(self, response: operations_pb2.Operation, metadata: Sequence[Tuple[str, Union[str, bytes]]]) -> Tuple[operations_pb2.Operation, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for update_instance

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the Instances server but before it is returned to user code.

        We recommend only using this `post_update_instance_with_metadata`
        interceptor in new development instead of the `post_update_instance` interceptor.
        When both interceptors are used, this `post_update_instance_with_metadata` interceptor runs after the
        `post_update_instance` interceptor. The (possibly modified) response returned by
        `post_update_instance` will be passed to
        `post_update_instance_with_metadata`.
        """
        return response, metadata


@dataclasses.dataclass
class InstancesRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: InstancesRestInterceptor


class InstancesRestTransport(_BaseInstancesRestTransport):
    """REST backend synchronous transport for Instances.

    The Cloud Run Instances API allows you to manage Cloud Run
    Instances.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1
    """

    def __init__(self, *,
            host: str = 'run.googleapis.com',
            credentials: Optional[ga_credentials.Credentials] = None,
            credentials_file: Optional[str] = None,
            scopes: Optional[Sequence[str]] = None,
            client_cert_source_for_mtls: Optional[Callable[[
                ], Tuple[bytes, bytes]]] = None,
            quota_project_id: Optional[str] = None,
            client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
            always_use_jwt_access: Optional[bool] = False,
            url_scheme: str = 'https',
            interceptor: Optional[InstancesRestInterceptor] = None,
            api_audience: Optional[str] = None,
            ) -> None:
        """Instantiate the transport.

       NOTE: This REST transport functionality is currently in a beta
       state (preview). We welcome your feedback via a GitHub issue in
       this library's repository. Thank you!

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'run.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): Deprecated. A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided. This argument will be
                removed in the next major version of this library.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            url_scheme=url_scheme,
            api_audience=api_audience
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST)
        self._operations_client: Optional[operations_v1.AbstractOperationsClient] = None
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or InstancesRestInterceptor()
        self._prep_wrapped_messages(client_info)

    @property
    def operations_client(self) -> operations_v1.AbstractOperationsClient:
        """Create the client designed to process long-running operations.

        This property caches on the instance; repeated calls return the same
        client.
        """
        # Only create a new client if we do not already have one.
        if self._operations_client is None:
            http_options: Dict[str, List[Dict[str, str]]] = {
            }

            rest_transport = operations_v1.OperationsRestTransport(
                    host=self._host,
                    # use the credentials which are saved
                    credentials=self._credentials,
                    scopes=self._scopes,
                    http_options=http_options,
                    path_prefix="v2")

            self._operations_client = operations_v1.AbstractOperationsClient(transport=rest_transport)

        # Return the client from cache.
        return self._operations_client

    class _CreateInstance(_BaseInstancesRestTransport._BaseCreateInstance, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.CreateInstance")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
                )
            return response

        def __call__(self,
                request: gcr_instance.CreateInstanceRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> operations_pb2.Operation:
            r"""Call the create instance method over HTTP.

            Args:
                request (~.gcr_instance.CreateInstanceRequest):
                    The request object.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options = _BaseInstancesRestTransport._BaseCreateInstance._get_http_options()

            request, metadata = self._interceptor.pre_create_instance(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseCreateInstance._get_transcoded_request(http_options, request)

            body = _BaseInstancesRestTransport._BaseCreateInstance._get_request_body_json(transcoded_request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseCreateInstance._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.CreateInstance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "CreateInstance",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._CreateInstance._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request, body)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_create_instance(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_create_instance_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.create_instance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "CreateInstance",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _DeleteInstance(_BaseInstancesRestTransport._BaseDeleteInstance, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.DeleteInstance")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                )
            return response

        def __call__(self,
                request: instance.DeleteInstanceRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> operations_pb2.Operation:
            r"""Call the delete instance method over HTTP.

            Args:
                request (~.instance.DeleteInstanceRequest):
                    The request object.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options = _BaseInstancesRestTransport._BaseDeleteInstance._get_http_options()

            request, metadata = self._interceptor.pre_delete_instance(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseDeleteInstance._get_transcoded_request(http_options, request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseDeleteInstance._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.DeleteInstance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "DeleteInstance",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._DeleteInstance._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_delete_instance(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_delete_instance_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.delete_instance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "DeleteInstance",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _GetIamPolicy(_BaseInstancesRestTransport._BaseGetIamPolicy, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.GetIamPolicy")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                )
            return response

        def __call__(self,
                request: iam_policy_pb2.GetIamPolicyRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> policy_pb2.Policy:
            r"""Call the get iam policy method over HTTP.

            Args:
                request (~.iam_policy_pb2.GetIamPolicyRequest):
                    The request object. Request message for ``GetIamPolicy`` method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.policy_pb2.Policy:
                    Defines an Identity and Access Management (IAM) policy.
                It is used to specify access control policies for Cloud
                Platform resources.

                A ``Policy`` is a collection of ``bindings``. A
                ``binding`` binds one or more ``members`` to a single
                ``role``. Members can be user accounts, service
                accounts, Google groups, and domains (such as G Suite).
                A ``role`` is a named list of permissions (defined by
                IAM or configured by users). A ``binding`` can
                optionally specify a ``condition``, which is a logic
                expression that further constrains the role binding
                based on attributes about the request and/or target
                resource.

                **JSON Example**

                ::

                    {
                      "bindings": [
                        {
                          "role": "roles/resourcemanager.organizationAdmin",
                          "members": [
                            "user:mike@example.com",
                            "group:admins@example.com",
                            "domain:google.com",
                            "serviceAccount:my-project-id@appspot.gserviceaccount.com"
                          ]
                        },
                        {
                          "role": "roles/resourcemanager.organizationViewer",
                          "members": ["user:eve@example.com"],
                          "condition": {
                            "title": "expirable access",
                            "description": "Does not grant access after Sep 2020",
                            "expression": "request.time <
                            timestamp('2020-10-01T00:00:00.000Z')",
                          }
                        }
                      ]
                    }

                **YAML Example**

                ::

                    bindings:
                    - members:
                      - user:mike@example.com
                      - group:admins@example.com
                      - domain:google.com
                      - serviceAccount:my-project-id@appspot.gserviceaccount.com
                      role: roles/resourcemanager.organizationAdmin
                    - members:
                      - user:eve@example.com
                      role: roles/resourcemanager.organizationViewer
                      condition:
                        title: expirable access
                        description: Does not grant access after Sep 2020
                        expression: request.time < timestamp('2020-10-01T00:00:00.000Z')

                For a description of IAM and its features, see the `IAM
                developer's
                guide <https://cloud.google.com/iam/docs>`__.

            """

            http_options = _BaseInstancesRestTransport._BaseGetIamPolicy._get_http_options()

            request, metadata = self._interceptor.pre_get_iam_policy(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseGetIamPolicy._get_transcoded_request(http_options, request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseGetIamPolicy._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.GetIamPolicy",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "GetIamPolicy",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._GetIamPolicy._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = policy_pb2.Policy()
            pb_resp = resp

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_get_iam_policy(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_get_iam_policy_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.get_iam_policy",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "GetIamPolicy",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _GetInstance(_BaseInstancesRestTransport._BaseGetInstance, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.GetInstance")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                )
            return response

        def __call__(self,
                request: instance.GetInstanceRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> instance.Instance:
            r"""Call the get instance method over HTTP.

            Args:
                request (~.instance.GetInstanceRequest):
                    The request object.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.instance.Instance:
                    A Cloud Run Instance represents a
                single group of containers running in a
                region.

            """

            http_options = _BaseInstancesRestTransport._BaseGetInstance._get_http_options()

            request, metadata = self._interceptor.pre_get_instance(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseGetInstance._get_transcoded_request(http_options, request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseGetInstance._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.GetInstance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "GetInstance",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._GetInstance._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = instance.Instance()
            pb_resp = instance.Instance.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_get_instance(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_get_instance_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = instance.Instance.to_json(response)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.get_instance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "GetInstance",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _ListInstances(_BaseInstancesRestTransport._BaseListInstances, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.ListInstances")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                )
            return response

        def __call__(self,
                request: instance.ListInstancesRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> instance.ListInstancesResponse:
            r"""Call the list instances method over HTTP.

            Args:
                request (~.instance.ListInstancesRequest):
                    The request object. Request message for retrieving a list
                of Instances.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.instance.ListInstancesResponse:
                    Response message containing a list of
                Instances.

            """

            http_options = _BaseInstancesRestTransport._BaseListInstances._get_http_options()

            request, metadata = self._interceptor.pre_list_instances(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseListInstances._get_transcoded_request(http_options, request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseListInstances._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.ListInstances",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "ListInstances",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._ListInstances._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = instance.ListInstancesResponse()
            pb_resp = instance.ListInstancesResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_list_instances(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_list_instances_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = instance.ListInstancesResponse.to_json(response)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.list_instances",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "ListInstances",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _SetIamPolicy(_BaseInstancesRestTransport._BaseSetIamPolicy, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.SetIamPolicy")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
                )
            return response

        def __call__(self,
                request: iam_policy_pb2.SetIamPolicyRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> policy_pb2.Policy:
            r"""Call the set iam policy method over HTTP.

            Args:
                request (~.iam_policy_pb2.SetIamPolicyRequest):
                    The request object. Request message for ``SetIamPolicy`` method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.policy_pb2.Policy:
                    Defines an Identity and Access Management (IAM) policy.
                It is used to specify access control policies for Cloud
                Platform resources.

                A ``Policy`` is a collection of ``bindings``. A
                ``binding`` binds one or more ``members`` to a single
                ``role``. Members can be user accounts, service
                accounts, Google groups, and domains (such as G Suite).
                A ``role`` is a named list of permissions (defined by
                IAM or configured by users). A ``binding`` can
                optionally specify a ``condition``, which is a logic
                expression that further constrains the role binding
                based on attributes about the request and/or target
                resource.

                **JSON Example**

                ::

                    {
                      "bindings": [
                        {
                          "role": "roles/resourcemanager.organizationAdmin",
                          "members": [
                            "user:mike@example.com",
                            "group:admins@example.com",
                            "domain:google.com",
                            "serviceAccount:my-project-id@appspot.gserviceaccount.com"
                          ]
                        },
                        {
                          "role": "roles/resourcemanager.organizationViewer",
                          "members": ["user:eve@example.com"],
                          "condition": {
                            "title": "expirable access",
                            "description": "Does not grant access after Sep 2020",
                            "expression": "request.time <
                            timestamp('2020-10-01T00:00:00.000Z')",
                          }
                        }
                      ]
                    }

                **YAML Example**

                ::

                    bindings:
                    - members:
                      - user:mike@example.com
                      - group:admins@example.com
                      - domain:google.com
                      - serviceAccount:my-project-id@appspot.gserviceaccount.com
                      role: roles/resourcemanager.organizationAdmin
                    - members:
                      - user:eve@example.com
                      role: roles/resourcemanager.organizationViewer
                      condition:
                        title: expirable access
                        description: Does not grant access after Sep 2020
                        expression: request.time < timestamp('2020-10-01T00:00:00.000Z')

                For a description of IAM and its features, see the `IAM
                developer's
                guide <https://cloud.google.com/iam/docs>`__.

            """

            http_options = _BaseInstancesRestTransport._BaseSetIamPolicy._get_http_options()

            request, metadata = self._interceptor.pre_set_iam_policy(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseSetIamPolicy._get_transcoded_request(http_options, request)

            body = _BaseInstancesRestTransport._BaseSetIamPolicy._get_request_body_json(transcoded_request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseSetIamPolicy._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.SetIamPolicy",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "SetIamPolicy",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._SetIamPolicy._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request, body)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = policy_pb2.Policy()
            pb_resp = resp

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_set_iam_policy(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_set_iam_policy_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.set_iam_policy",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "SetIamPolicy",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _StartInstance(_BaseInstancesRestTransport._BaseStartInstance, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.StartInstance")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
                )
            return response

        def __call__(self,
                request: instance.StartInstanceRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> operations_pb2.Operation:
            r"""Call the start instance method over HTTP.

            Args:
                request (~.instance.StartInstanceRequest):
                    The request object. Request message for starting an
                Instance.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options = _BaseInstancesRestTransport._BaseStartInstance._get_http_options()

            request, metadata = self._interceptor.pre_start_instance(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseStartInstance._get_transcoded_request(http_options, request)

            body = _BaseInstancesRestTransport._BaseStartInstance._get_request_body_json(transcoded_request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseStartInstance._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.StartInstance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "StartInstance",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._StartInstance._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request, body)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_start_instance(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_start_instance_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.start_instance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "StartInstance",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _StopInstance(_BaseInstancesRestTransport._BaseStopInstance, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.StopInstance")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
                )
            return response

        def __call__(self,
                request: instance.StopInstanceRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> operations_pb2.Operation:
            r"""Call the stop instance method over HTTP.

            Args:
                request (~.instance.StopInstanceRequest):
                    The request object. Request message for deleting an
                Instance.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options = _BaseInstancesRestTransport._BaseStopInstance._get_http_options()

            request, metadata = self._interceptor.pre_stop_instance(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseStopInstance._get_transcoded_request(http_options, request)

            body = _BaseInstancesRestTransport._BaseStopInstance._get_request_body_json(transcoded_request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseStopInstance._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.StopInstance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "StopInstance",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._StopInstance._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request, body)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_stop_instance(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_stop_instance_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.stop_instance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "StopInstance",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _TestIamPermissions(_BaseInstancesRestTransport._BaseTestIamPermissions, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.TestIamPermissions")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
                )
            return response

        def __call__(self,
                request: iam_policy_pb2.TestIamPermissionsRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> iam_policy_pb2.TestIamPermissionsResponse:
            r"""Call the test iam permissions method over HTTP.

            Args:
                request (~.iam_policy_pb2.TestIamPermissionsRequest):
                    The request object. Request message for ``TestIamPermissions`` method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.iam_policy_pb2.TestIamPermissionsResponse:
                    Response message for ``TestIamPermissions`` method.
            """

            http_options = _BaseInstancesRestTransport._BaseTestIamPermissions._get_http_options()

            request, metadata = self._interceptor.pre_test_iam_permissions(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseTestIamPermissions._get_transcoded_request(http_options, request)

            body = _BaseInstancesRestTransport._BaseTestIamPermissions._get_request_body_json(transcoded_request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseTestIamPermissions._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.TestIamPermissions",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "TestIamPermissions",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._TestIamPermissions._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request, body)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = iam_policy_pb2.TestIamPermissionsResponse()
            pb_resp = resp

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_test_iam_permissions(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_test_iam_permissions_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.test_iam_permissions",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "TestIamPermissions",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _UpdateInstance(_BaseInstancesRestTransport._BaseUpdateInstance, InstancesRestStub):
        def __hash__(self):
            return hash("InstancesRestTransport.UpdateInstance")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None):

            uri = transcoded_request['uri']
            method = transcoded_request['method']
            headers = dict(metadata)
            headers['Content-Type'] = 'application/json'
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
                )
            return response

        def __call__(self,
                request: gcr_instance.UpdateInstanceRequest, *,
                retry: OptionalRetry=gapic_v1.method.DEFAULT,
                timeout: Optional[float]=None,
                metadata: Sequence[Tuple[str, Union[str, bytes]]]=(),
                ) -> operations_pb2.Operation:
            r"""Call the update instance method over HTTP.

            Args:
                request (~.gcr_instance.UpdateInstanceRequest):
                    The request object. Request message for updating an
                Instance.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options = _BaseInstancesRestTransport._BaseUpdateInstance._get_http_options()

            request, metadata = self._interceptor.pre_update_instance(request, metadata)
            transcoded_request = _BaseInstancesRestTransport._BaseUpdateInstance._get_transcoded_request(http_options, request)

            body = _BaseInstancesRestTransport._BaseUpdateInstance._get_request_body_json(transcoded_request)

            # Jsonify the query params
            query_params = _BaseInstancesRestTransport._BaseUpdateInstance._get_query_params_json(transcoded_request)

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                request_url = "{host}{uri}".format(host=self._host, uri=transcoded_request['uri'])
                method = transcoded_request['method']
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                  "payload": request_payload,
                  "requestMethod": method,
                  "requestUrl": request_url,
                  "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.run_v2.InstancesClient.UpdateInstance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "UpdateInstance",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = InstancesRestTransport._UpdateInstance._get_response(self._host, metadata, query_params, self._session, timeout, transcoded_request, body)

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_update_instance(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_update_instance_with_metadata(resp, response_metadata)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(logging.DEBUG):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                "payload": response_payload,
                "headers":  dict(response.headers),
                "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.run_v2.InstancesClient.update_instance",
                    extra = {
                        "serviceName": "google.cloud.run.v2.Instances",
                        "rpcName": "UpdateInstance",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    @property
    def create_instance(self) -> Callable[
            [gcr_instance.CreateInstanceRequest],
            operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._CreateInstance(self._session, self._host, self._interceptor) # type: ignore

    @property
    def delete_instance(self) -> Callable[
            [instance.DeleteInstanceRequest],
            operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._DeleteInstance(self._session, self._host, self._interceptor) # type: ignore

    @property
    def get_iam_policy(self) -> Callable[
            [iam_policy_pb2.GetIamPolicyRequest],
            policy_pb2.Policy]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetIamPolicy(self._session, self._host, self._interceptor) # type: ignore

    @property
    def get_instance(self) -> Callable[
            [instance.GetInstanceRequest],
            instance.Instance]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetInstance(self._session, self._host, self._interceptor) # type: ignore

    @property
    def list_instances(self) -> Callable[
            [instance.ListInstancesRequest],
            instance.ListInstancesResponse]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ListInstances(self._session, self._host, self._interceptor) # type: ignore

    @property
    def set_iam_policy(self) -> Callable[
            [iam_policy_pb2.SetIamPolicyRequest],
            policy_pb2.Policy]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._SetIamPolicy(self._session, self._host, self._interceptor) # type: ignore

    @property
    def start_instance(self) -> Callable[
            [instance.StartInstanceRequest],
            operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._StartInstance(self._session, self._host, self._interceptor) # type: ignore

    @property
    def stop_instance(self) -> Callable[
            [instance.StopInstanceRequest],
            operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._StopInstance(self._session, self._host, self._interceptor) # type: ignore

    @property
    def test_iam_permissions(self) -> Callable[
            [iam_policy_pb2.TestIamPermissionsRequest],
            iam_policy_pb2.TestIamPermissionsResponse]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._TestIamPermissions(self._session, self._host, self._interceptor) # type: ignore

    @property
    def update_instance(self) -> Callable[
            [gcr_instance.UpdateInstanceRequest],
            operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._UpdateInstance(self._session, self._host, self._interceptor) # type: ignore

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__=(
    'InstancesRestTransport',
)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Datastream connection profiles API."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.datastream import exceptions as ds_exceptions
from googlecloudsdk.api_lib.datastream import util
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.calliope.arg_parsers import HostPort
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import resources
from googlecloudsdk.core import yaml
from googlecloudsdk.core.console import console_io


def GetConnectionProfileURI(resource):
  connection_profile = resources.REGISTRY.ParseRelativeName(
      resource.name,
      collection='datastream.projects.locations.connectionProfiles')
  return connection_profile.SelfLink()


class ConnectionProfilesClient:
  """Client for connection profiles service in the API."""

  def __init__(self, client=None, messages=None, location=None):
    self._client = client or util.GetClientInstance(location=location)
    self._messages = messages or util.GetMessagesModule()
    self._service = self._client.projects_locations_connectionProfiles
    self._resource_parser = util.GetResourceParser()

  def _ValidateArgs(self, args):
    self._ValidateSslConfigArgs(args)

  def _ValidateSslConfigArgs(self, args):
    """Validates Format of all SSL config args."""
    self._ValidateCertificateFormat(args.ca_certificate, 'CA certificate')
    self._ValidateCertificateFormat(args.client_certificate,
                                    'client certificate')
    self._ValidateCertificateFormat(args.client_key, 'client key')

    # Validation for all Postgresql SSL config fields.
    self._ValidateCertificateFormat(
        args.postgresql_ca_certificate, 'Postgresql CA certificate'
    )
    self._ValidateCertificateFormat(
        args.postgresql_client_certificate, 'Postgresql client certificate'
    )
    self._ValidateCertificateFormat(
        args.postgresql_client_key, 'Postgresql client private key'
    )

    # Validation for Oracle SSL config fields.
    self._ValidateCertificateFormat(
        args.oracle_ca_certificate, 'Oracle CA certificate'
    )

  def _ValidateCertificateFormat(self, certificate, name):
    if not certificate:
      return True
    cert = certificate.strip()
    cert_lines = cert.split('\n')
    if (not cert_lines[0].startswith('-----') or
        not cert_lines[-1].startswith('-----')):
      raise exceptions.InvalidArgumentException(
          name,
          'The certificate does not appear to be in PEM format: \n{0}'.format(
              cert))

  def _GetSslConfig(self, args):
    return self._messages.MysqlSslConfig(
        clientKey=args.client_key,
        clientCertificate=args.client_certificate,
        caCertificate=args.ca_certificate)

  def _GetMySqlProfile(self, args):
    ssl_config = self._GetSslConfig(args)
    return self._messages.MysqlProfile(
        hostname=args.mysql_hostname,
        port=args.mysql_port,
        username=args.mysql_username,
        password=args.mysql_password,
        secretManagerStoredPassword=args.mysql_secret_manager_stored_password,
        sslConfig=ssl_config)

  def _GetOracleProfile(self, args):
    ssl_config = self._GetOracleSslConfig(args)
    return self._messages.OracleProfile(
        hostname=args.oracle_hostname,
        port=args.oracle_port,
        username=args.oracle_username,
        password=args.oracle_password,
        secretManagerStoredPassword=args.oracle_secret_manager_stored_password,
        databaseService=args.database_service,
        oracleSslConfig=ssl_config)

  def _GetOracleSslConfig(self, args):
    """Returns a OracleSslConfig message based on the given args."""
    return self._messages.OracleSslConfig(
        caCertificate=args.oracle_ca_certificate,
        serverCertificateDistinguishedName=args.oracle_server_certificate_distinguished_name,
    )

  def _GetPostgresqlSslConfig(self, args):
    """Returns a PostgresqlSslConfig message based on the given args."""
    if args.postgresql_client_certificate or args.postgresql_client_key:
      return self._messages.PostgresqlSslConfig(
          serverAndClientVerification=self._messages.ServerAndClientVerification(
              clientCertificate=args.postgresql_client_certificate,
              clientKey=args.postgresql_client_key,
              caCertificate=args.postgresql_ca_certificate,
              serverCertificateHostname=args.postgresql_server_certificate_hostname,
          )
      )

    if args.postgresql_ca_certificate:
      return self._messages.PostgresqlSslConfig(
          serverVerification=self._messages.ServerVerification(
              caCertificate=args.postgresql_ca_certificate,
              serverCertificateHostname=args.postgresql_server_certificate_hostname,
          )
      )

    return None

  def _GetPostgresqlProfile(self, args):
    ssl_config = self._GetPostgresqlSslConfig(args)
    return self._messages.PostgresqlProfile(
        hostname=args.postgresql_hostname,
        port=args.postgresql_port,
        username=args.postgresql_username,
        password=args.postgresql_password,
        secretManagerStoredPassword=args.postgresql_secret_manager_stored_password,
        database=args.postgresql_database,
        sslConfig=ssl_config)

  def _GetSqlServerProfile(self, args):
    return self._messages.SqlServerProfile(
        hostname=args.sqlserver_hostname,
        port=args.sqlserver_port,
        username=args.sqlserver_username,
        password=args.sqlserver_password,
        secretManagerStoredPassword=args.sqlserver_secret_manager_stored_password,
        database=args.sqlserver_database,
    )

  def _GetSalesforceProfile(self, args):
    if args.salesforce_oauth2_client_id:
      return self._messages.SalesforceProfile(
          domain=args.salesforce_domain,
          oauth2ClientCredentials=self._messages.Oauth2ClientCredentials(
              clientId=args.salesforce_oauth2_client_id,
              clientSecret=args.salesforce_oauth2_client_secret,
              secretManagerStoredClientSecret=args.salesforce_secret_manager_stored_oauth2_client_secret,
          ),
      )
    else:
      return self._messages.SalesforceProfile(
          domain=args.salesforce_domain,
          userCredentials=self._messages.UserCredentials(
              username=args.salesforce_username,
              password=args.salesforce_password,
              secretManagerStoredPassword=args.salesforce_secret_manager_stored_password,
              securityToken=args.salesforce_security_token,
              secretManagerStoredSecurityToken=args.salesforce_secret_manager_stored_security_token,
          ),
      )

  def _GetSpannerProfile(self, args):
    return self._messages.SpannerProfile(
        database=args.spanner_database,
        host=args.spanner_host,
    )

  def _GetGCSProfile(self, args, release_track):
    # TODO(b/207467120): remove bucket_name arg check.
    if release_track == base.ReleaseTrack.BETA:
      bucket = args.bucket_name
    else:
      bucket = args.bucket

    gcs_profile = self._messages.GcsProfile(bucket=bucket)
    gcs_profile.rootPath = args.root_path if args.root_path else '/'
    return gcs_profile

  def _GetMongodbProfile(self, args):
    """Returns the MongoDB profile message based on the given args."""
    addresses = []
    for host_address in args.mongodb_host_addresses:
      if args.mongodb_srv_connection_format:
        addresses.append(
            self._messages.HostAddress(hostname=host_address)
        )
      else:
        hostport = HostPort.Parse(host_address)
        addresses.append(
            self._messages.HostAddress(
                hostname=hostport.host, port=int(hostport.port)
            )
        )
    profile = self._messages.MongodbProfile(
        hostAddresses=addresses,
        username=args.mongodb_username,
        replicaSet=args.mongodb_replica_set,
        password=args.mongodb_password,
        secretManagerStoredPassword=args.mongodb_secret_manager_stored_password,
    )
    if (
        args.mongodb_direct_connection
        and not args.mongodb_standard_connection_format
    ):
      raise exceptions.InvalidArgumentException(
          'mongodb-direct-connection',
          'mongodb direct connection can only be used with the standard'
          ' connection format.',
      )
    if args.mongodb_srv_connection_format:
      profile.srvConnectionFormat = {}
    if args.mongodb_standard_connection_format:
      profile.standardConnectionFormat = (
          self._messages.StandardConnectionFormat(
              directConnection=args.mongodb_direct_connection
          )
      )
    if args.mongodb_tls:
      profile.sslConfig = {}
      if args.mongodb_ca_certificate:
        profile.sslConfig.caCertificate = args.mongodb_ca_certificate
    if args.mongodb_additional_options:
      additional_options = []
      for key, value in args.mongodb_additional_options.items():
        additional_options.append(
            self._messages.MongodbProfile.AdditionalOptionsValue.AdditionalProperty(
                key=key, value=value
            )
        )
      profile.additionalOptions = (
          self._messages.MongodbProfile.AdditionalOptionsValue(
              additionalProperties=additional_options
          )
      )
    return profile

  def _GetDataverseProfile(self, args):
    """Returns the Dataverse profile message based on the given args."""
    client_secret = self._messages.Secret()
    if args.dataverse_oauth_client_secret:
      client_secret.rawValue = args.dataverse_oauth_client_secret
    elif args.dataverse_secret_manager_stored_oauth_client_secret:
      client_secret.secretVersion = (
          args.dataverse_secret_manager_stored_oauth_client_secret
      )

    return self._messages.DataverseProfile(
        environmentUrl=args.dataverse_environment_url,
        tenantId=args.dataverse_tenant_id,
        oauthClientCredentials=self._messages.OauthClientCredentials(
            clientId=args.dataverse_oauth_client_id,
            clientSecret=client_secret,
        ),
    )

  def _GetSalesforceMarketingCloudProfile(self, args):
    """Returns the Salesforce Marketing Cloud profile message based on the given args."""
    client_secret = self._messages.Secret()
    if args.salesforce_marketing_cloud_oauth_client_secret:
      client_secret.rawValue = (
          args.salesforce_marketing_cloud_oauth_client_secret
      )
    elif (
        args.salesforce_marketing_cloud_secret_manager_stored_oauth_client_secret
    ):
      client_secret.secretVersion = (
          args.salesforce_marketing_cloud_secret_manager_stored_oauth_client_secret
      )

    return self._messages.SalesforceMarketingCloudProfile(
        subdomain=args.salesforce_marketing_cloud_subdomain,
        oauthClientCredentials=self._messages.OauthClientCredentials(
            clientId=args.salesforce_marketing_cloud_oauth_client_id,
            clientSecret=client_secret,
        ),
    )

  def _GetServiceNowProfile(self, args):
    """Returns the ServiceNow profile message based on the given args."""
    profile = self._messages.ServiceNowProfile(
        instance=args.servicenow_instance
    )
    if args.servicenow_oauth_client_id:
      client_secret = self._messages.Secret()
      if args.servicenow_oauth_client_secret:
        client_secret.rawValue = args.servicenow_oauth_client_secret
      elif args.servicenow_secret_manager_stored_oauth_client_secret:
        client_secret.secretVersion = (
            args.servicenow_secret_manager_stored_oauth_client_secret
        )
      profile.oauthClientCredentials = self._messages.OauthClientCredentials(
          clientId=args.servicenow_oauth_client_id,
          clientSecret=client_secret,
      )
    elif args.servicenow_username:
      password = self._messages.Secret()
      if args.servicenow_password:
        password.rawValue = args.servicenow_password
      elif args.servicenow_secret_manager_stored_password:
        password.secretVersion = args.servicenow_secret_manager_stored_password
      profile.userPasswordCredentials = self._messages.UserPasswordCredentials(
          username=args.servicenow_username,
          password=password,
      )
    return profile

  def _ParseSslConfig(self, data):
    return self._messages.MysqlSslConfig(
        clientKey=data.get('client_key'),
        clientCertificate=data.get('client_certificate'),
        caCertificate=data.get('ca_certificate'))

  def _ParseMySqlProfile(self, data):
    if not data:
      return {}
    ssl_config = self._ParseSslConfig(data)
    return self._messages.MysqlProfile(
        hostname=data.get('hostname'),
        port=data.get('port'),
        username=data.get('username'),
        password=data.get('password'),
        sslConfig=ssl_config)

  def _ParseOracleProfile(self, data):
    if not data:
      return {}
    return self._messages.OracleProfile(
        hostname=data.get('hostname'),
        port=data.get('port'),
        username=data.get('username'),
        password=data.get('password'),
        databaseService=data.get('database_service'))

  def _ParsePostgresqlProfile(self, data):
    if not data:
      return {}
    return self._messages.PostgresqlProfile(
        hostname=data.get('hostname'),
        port=data.get('port'),
        username=data.get('username'),
        password=data.get('password'),
        database=data.get('database'))

  def _ParseSqlServerProfile(self, data):
    if not data:
      return {}
    return self._messages.SqlServerProfile(
        hostname=data.get('hostname'),
        port=data.get('port'),
        username=data.get('username'),
        password=data.get('password'),
        database=data.get('database'),
    )

  def _ParseSpannerProfile(self, data):
    if not data:
      return {}
    return self._messages.SpannerProfile(
        database=data.get('database'),
        host=data.get('host'),
    )

  def _ParseGCSProfile(self, data):
    if not data:
      return {}
    return self._messages.GcsProfile(
        bucket=data.get('bucket_name'), rootPath=data.get('root_path'))

  def _GetForwardSshTunnelConnectivity(self, args):
    return self._messages.ForwardSshTunnelConnectivity(
        hostname=args.forward_ssh_hostname,
        port=args.forward_ssh_port,
        username=args.forward_ssh_username,
        privateKey=args.forward_ssh_private_key,
        password=args.forward_ssh_password)

  def _GetConnectionProfile(self, cp_type, connection_profile_id, args,
                            release_track):
    """Returns a connection profile according to type."""
    labels = labels_util.ParseCreateArgs(
        args, self._messages.ConnectionProfile.LabelsValue)
    connection_profile_obj = self._messages.ConnectionProfile(
        name=connection_profile_id, labels=labels,
        displayName=args.display_name)

    if cp_type == 'MYSQL':
      connection_profile_obj.mysqlProfile = self._GetMySqlProfile(args)
    elif cp_type == 'ORACLE':
      connection_profile_obj.oracleProfile = self._GetOracleProfile(args)
    elif cp_type == 'POSTGRESQL':
      connection_profile_obj.postgresqlProfile = self._GetPostgresqlProfile(
          args)
    elif cp_type == 'SQLSERVER':
      connection_profile_obj.sqlServerProfile = self._GetSqlServerProfile(args)
    elif cp_type == 'GOOGLE-CLOUD-STORAGE':
      connection_profile_obj.gcsProfile = self._GetGCSProfile(
          args, release_track)
    elif cp_type == 'BIGQUERY':
      connection_profile_obj.bigqueryProfile = self._messages.BigQueryProfile()
    elif cp_type == 'SALESFORCE':
      connection_profile_obj.salesforceProfile = self._GetSalesforceProfile(
          args
      )
    elif cp_type == 'DATAVERSE':
      connection_profile_obj.dataverseProfile = self._GetDataverseProfile(args)
    elif cp_type == 'SALESFORCE-MARKETING-CLOUD':
      connection_profile_obj.salesforceMarketingCloudProfile = (
          self._GetSalesforceMarketingCloudProfile(args)
      )
    elif cp_type == 'SERVICENOW':
      connection_profile_obj.serviceNowProfile = self._GetServiceNowProfile(
          args
      )
    elif cp_type == 'SPANNER':
      connection_profile_obj.spannerProfile = self._GetSpannerProfile(
          args
      )
    elif cp_type == 'MONGODB':
      connection_profile_obj.mongodbProfile = self._GetMongodbProfile(args)
    else:
      raise exceptions.InvalidArgumentException(
          cp_type,
          'The connection profile type {0} is either unknown or not supported'
          ' yet.'.format(cp_type),
      )

    # TODO(b/207467120): deprecate BETA client.
    if release_track == base.ReleaseTrack.BETA:
      private_connectivity_ref = args.CONCEPTS.private_connection_name.Parse()
    else:
      private_connectivity_ref = args.CONCEPTS.private_connection.Parse()

    if private_connectivity_ref:
      connection_profile_obj.privateConnectivity = (
          self._messages.PrivateConnectivity(
              privateConnection=private_connectivity_ref.RelativeName()
          )
      )
    elif args.forward_ssh_hostname:
      connection_profile_obj.forwardSshConnectivity = (
          self._GetForwardSshTunnelConnectivity(args)
      )
    elif args.static_ip_connectivity:
      connection_profile_obj.staticServiceIpConnectivity = {}

    return connection_profile_obj

  def _ParseConnectionProfileObjectFile(
      self, connection_profile_object_file, release_track
  ):
    """Parses a connection-profile-file into the ConnectionProfile message."""
    if release_track != base.ReleaseTrack.BETA:
      return util.ParseMessageAndValidateSchema(
          connection_profile_object_file,
          'ConnectionProfile',
          self._messages.ConnectionProfile,
      )

    data = console_io.ReadFromFileOrStdin(
        connection_profile_object_file, binary=False)
    try:
      connection_profile_data = yaml.load(data)
    except Exception as e:
      raise ds_exceptions.ParseError('Cannot parse YAML:[{0}]'.format(e))

    display_name = connection_profile_data.get('display_name')
    labels = connection_profile_data.get('labels')
    connection_profile_msg = self._messages.ConnectionProfile(
        displayName=display_name,
        labels=labels)

    oracle_profile = self._ParseOracleProfile(
        connection_profile_data.get('oracle_profile', {}))
    mysql_profile = self._ParseMySqlProfile(
        connection_profile_data.get('mysql_profile', {}))
    postgresql_profile = self._ParsePostgresqlProfile(
        connection_profile_data.get('postgresql_profile', {}))
    sqlserver_profile = self._ParseSqlServerProfile(
        connection_profile_data.get('sqlserver_profile', {}))
    spanner_profile = self._ParseSpannerProfile(
        connection_profile_data.get('spanner_profile', {}))

    gcs_profile = self._ParseGCSProfile(
        connection_profile_data.get('gcs_profile', {}))
    if oracle_profile:
      connection_profile_msg.oracleProfile = oracle_profile
    elif mysql_profile:
      connection_profile_msg.mysqlProfile = mysql_profile
    elif postgresql_profile:
      connection_profile_msg.postgresqlProfile = postgresql_profile
    elif sqlserver_profile:
      connection_profile_msg.sqlServerProfile = sqlserver_profile
    elif spanner_profile:
      connection_profile_msg.spannerProfile = spanner_profile
    elif gcs_profile:
      connection_profile_msg.gcsProfile = gcs_profile

    if 'static_service_ip_connectivity' in connection_profile_data:
      connection_profile_msg.staticServiceIpConnectivity = (
          connection_profile_data.get('static_service_ip_connectivity')
      )
    elif 'forward_ssh_connectivity' in connection_profile_data:
      connection_profile_msg.forwardSshConnectivity = (
          connection_profile_data.get('forward_ssh_connectivity')
      )
    elif 'private_connectivity' in connection_profile_data:
      connection_profile_msg.privateConnectivity = connection_profile_data.get(
          'private_connectivity'
      )
    # Spanner profile does not require connectivity method.
    elif spanner_profile:
      pass
    else:
      raise ds_exceptions.ParseError(
          'Cannot parse YAML: missing connectivity method.'
      )

    return connection_profile_msg

  def _UpdateForwardSshTunnelConnectivity(
      self, connection_profile, args, update_fields
  ):
    """Updates Forward SSH tunnel connectivity config."""
    if args.IsSpecified('forward_ssh_hostname'):
      connection_profile.forwardSshConnectivity.hostname = (
          args.forward_ssh_hostname
      )
      update_fields.append('forwardSshConnectivity.hostname')
    if args.IsSpecified('forward_ssh_port'):
      connection_profile.forwardSshConnectivity.port = args.forward_ssh_port
      update_fields.append('forwardSshConnectivity.port')
    if args.IsSpecified('forward_ssh_username'):
      connection_profile.forwardSshConnectivity.username = (
          args.forward_ssh_username
      )
      update_fields.append('forwardSshConnectivity.username')
    if args.IsSpecified('forward_ssh_private_key'):
      connection_profile.forwardSshConnectivity.privateKey = (
          args.forward_ssh_private_key
      )
      update_fields.append('forwardSshConnectivity.privateKey')
    if args.IsSpecified('forward_ssh_password'):
      connection_profile.forwardSshConnectivity.privateKey = (
          args.forward_ssh_password
      )
      update_fields.append('forwardSshConnectivity.password')

  def _UpdateGCSProfile(
      self, connection_profile, release_track, args, update_fields
  ):
    """Updates GOOGLE CLOUD STORAGE connection profile."""
    # TODO(b/207467120): remove bucket_name arg check.
    if release_track == base.ReleaseTrack.BETA and args.IsSpecified(
        'bucket_name'
    ):
      connection_profile.gcsProfile.bucket = args.bucket_name
      update_fields.append('gcsProfile.bucket')
    if release_track == base.ReleaseTrack.GA and args.IsSpecified('bucket'):
      connection_profile.gcsProfile.bucket = args.bucket
      update_fields.append('gcsProfile.bucket')
    if args.IsSpecified('root_path'):
      connection_profile.gcsProfile.rootPath = args.root_path
      update_fields.append('gcsProfile.rootPath')

  def _UpdateOracleProfile(self,
                           connection_profile,
                           args,
                           update_fields):
    """Updates Oracle connection profile."""
    if args.IsSpecified('oracle_hostname'):
      connection_profile.oracleProfile.hostname = args.oracle_hostname
      update_fields.append('oracleProfile.hostname')
    if args.IsSpecified('oracle_port'):
      connection_profile.oracleProfile.port = args.oracle_port
      update_fields.append('oracleProfile.port')
    if args.IsSpecified('oracle_username'):
      connection_profile.oracleProfile.username = args.oracle_username
      update_fields.append('oracleProfile.username')
    if args.IsSpecified('oracle_password') or args.IsSpecified(
        'oracle_secret_manager_stored_password'
    ):
      connection_profile.oracleProfile.password = args.oracle_password
      connection_profile.oracleProfile.secretManagerStoredPassword = (
          args.oracle_secret_manager_stored_password
      )
      update_fields.append('oracleProfile.password')
      update_fields.append('oracleProfile.secretManagerStoredPassword')
    if args.IsSpecified('database_service'):
      connection_profile.oracleProfile.databaseService = args.database_service
      update_fields.append('oracleProfile.databaseService')

  def _UpdateMysqlSslConfig(self, connection_profile, args, update_fields):
    """Updates Mysql SSL config."""
    if args.IsSpecified('client_key'):
      connection_profile.mysqlProfile.sslConfig.clientKey = args.client_key
      update_fields.append('mysqlProfile.sslConfig.clientKey')
    if args.IsSpecified('client_certificate'):
      connection_profile.mysqlProfile.sslConfig.clientCertificate = (
          args.client_certificate
      )
      update_fields.append('mysqlProfile.sslConfig.clientCertificate')
    if args.IsSpecified('ca_certificate'):
      connection_profile.mysqlProfile.sslConfig.caCertificate = (
          args.ca_certificate
      )
      update_fields.append('mysqlProfile.sslConfig.caCertificate')

  def _UpdateMySqlProfile(self, connection_profile, args, update_fields):
    """Updates MySQL connection profile."""
    if args.IsSpecified('mysql_hostname'):
      connection_profile.mysqlProfile.hostname = args.mysql_hostname
      update_fields.append('mysqlProfile.hostname')
    if args.IsSpecified('mysql_port'):
      connection_profile.mysqlProfile.port = args.mysql_port
      update_fields.append('mysqlProfile.port')
    if args.IsSpecified('mysql_username'):
      connection_profile.mysqlProfile.username = args.mysql_username
      update_fields.append('mysqlProfile.username')
    if args.IsSpecified('mysql_password') or args.IsSpecified(
        'mysql_secret_manager_stored_password'
    ):
      connection_profile.mysqlProfile.password = args.mysql_password
      connection_profile.mysqlProfile.secretManagerStoredPassword = (
          args.mysql_secret_manager_stored_password
      )
      update_fields.append('mysqlProfile.password')
      update_fields.append('mysqlProfile.secretManagerStoredPassword')

    self._UpdateMysqlSslConfig(connection_profile, args, update_fields)

  def _UpdatePostgresqlSslConfig(self, connection_profile, args, update_fields):
    """Updates Postgresql SSL config."""
    if args.IsSpecified('postgresql_client_certificate'):
      connection_profile.postgresqlProfile.sslConfig.serverAndClientVerification.clientCertificate = (
          args.postgresql_client_certificate
      )
      update_fields.append(
          'postgresqlProfile.sslConfig.serverAndClientVerification.clientCertificate'
      )

    if args.IsSpecified('postgresql_client_key'):
      connection_profile.postgresqlProfile.sslConfig.serverAndClientVerification.clientKey = (
          args.postgresql_client_key
      )
      update_fields.append(
          'postgresqlProfile.sslConfig.serverAndClientVerification.clientKey'
      )

    if args.IsSpecified('postgresql_ca_certificate'):
      if connection_profile.postgresqlProfile.sslConfig.serverAndClientVerification:
        connection_profile.postgresqlProfile.sslConfig.serverAndClientVerification.caCertificate = (
            args.postgresql_ca_certificate
        )
        update_fields.append(
            'postgresqlProfile.sslConfig.serverAndClientVerification.caCertificate'
        )
      else:
        connection_profile.postgresqlProfile.sslConfig.serverVerification.caCertificate = (
            args.postgresql_ca_certificate
        )
        update_fields.append(
            'postgresqlProfile.sslConfig.serverVerification.caCertificate'
        )
    if args.IsSpecified('postgresql_server_certificate_hostname'):
      if (
          connection_profile.postgresqlProfile.sslConfig.serverAndClientVerification
      ):
        connection_profile.postgresqlProfile.sslConfig.serverAndClientVerification.serverCertificateHostname = (
            args.postgresql_server_certificate_hostname
        )
        update_fields.append(
            'postgresqlProfile.sslConfig.serverAndClientVerification.serverCertificateHostname'
        )
      else:
        connection_profile.postgresqlProfile.sslConfig.serverVerification.serverCertificateHostname = (
            args.postgresql_server_certificate_hostname
        )
        update_fields.append(
            'postgresqlProfile.sslConfig.serverVerification.serverCertificateHostname'
        )

  def _UpdatePostgresqlProfile(self, connection_profile, args, update_fields):
    """Updates Postgresql connection profile."""
    if args.IsSpecified('postgresql_hostname'):
      connection_profile.postgresqlProfile.hostname = args.postgresql_hostname
      update_fields.append('postgresqlProfile.hostname')
    if args.IsSpecified('postgresql_port'):
      connection_profile.postgresqlProfile.port = args.postgresql_port
      update_fields.append('postgresqlProfile.port')
    if args.IsSpecified('postgresql_username'):
      connection_profile.postgresqlProfile.username = args.postgresql_username
      update_fields.append('postgresqlProfile.username')
    if args.IsSpecified('postgresql_password') or args.IsSpecified(
        'postgresql_secret_manager_stored_password'
    ):
      connection_profile.postgresqlProfile.password = args.postgresql_password
      connection_profile.postgresqlProfile.secretManagerStoredPassword = (
          args.postgresql_secret_manager_stored_password
      )
      update_fields.append('postgresqlProfile.password')
      update_fields.append('postgresqlProfile.secretManagerStoredPassword')
    if args.IsSpecified('postgresql_database'):
      connection_profile.postgresqlProfile.database = args.postgresql_database
      update_fields.append('postgresqlProfile.database')

    self._UpdatePostgresqlSslConfig(connection_profile, args, update_fields)

  def _UpdateSqlServerProfile(self, connection_profile, args, update_fields):
    """Updates SqlServer connection profile."""
    if args.IsSpecified('sqlserver_hostname'):
      connection_profile.sqlServerProfile.hostname = args.sqlserver_hostname
      update_fields.append('sqlServerProfile.hostname')
    if args.IsSpecified('sqlserver_port'):
      connection_profile.sqlServerProfile.port = args.sqlserver_port
      update_fields.append('sqlServerProfile.port')
    if args.IsSpecified('sqlserver_username'):
      connection_profile.sqlServerProfile.username = args.sqlserver_username
      update_fields.append('sqlServerProfile.username')
    if args.IsSpecified('sqlserver_password') or args.IsSpecified(
        'sqlserver_secret_manager_stored_password'
    ):
      connection_profile.sqlServerProfile.password = args.sqlserver_password
      connection_profile.sqlServerProfile.secretManagerStoredPassword = (
          args.sqlserver_secret_manager_stored_password
      )
      update_fields.append('sqlServerProfile.password')
      update_fields.append('sqlServerProfile.secretManagerStoredPassword')
    if args.IsSpecified('sqlserver_database'):
      connection_profile.sqlServerProfile.database = args.sqlserver_database
      update_fields.append('sqlServerProfile.database')

  def _UpdateSalesforceProfile(self, connection_profile, args, update_fields):
    """Updates Salesforce connection profile."""
    if args.IsSpecified('salesforce_domain'):
      connection_profile.salesforceProfile.domain = args.salesforce_domain
      update_fields.append('salesforceProfile.domain')
    if args.IsSpecified('salesforce_username'):
      connection_profile.salesforceProfile.userCredentials.username = (
          args.salesforce_username
      )
      update_fields.append('salesforceProfile.userCredentials.username')
    if args.IsSpecified('salesforce_password') or args.IsSpecified(
        'salesforce_secret_manager_stored_password'
    ):
      connection_profile.salesforceProfile.userCredentials.password = (
          args.salesforce_password
      )
      connection_profile.salesforceProfile.userCredentials.secretManagerStoredPassword = (
          args.salesforce_secret_manager_stored_password
      )
      update_fields.append('salesforceProfile.userCredentials.password')
      update_fields.append(
          'salesforceProfile.userCredentials.secretManagerStoredPassword'
      )

    if args.IsSpecified('salesforce_security_token') or args.IsSpecified(
        'salesforce_secret_manager_stored_security_token'
    ):
      connection_profile.salesforceProfile.userCredentials.securityToken = (
          args.salesforce_security_token
      )
      connection_profile.salesforceProfile.userCredentials.secretManagerStoredSecurityToken = (
          args.salesforce_secret_manager_stored_security_token
      )
      update_fields.append('salesforceProfile.userCredentials.securityToken')
      update_fields.append(
          'salesforceProfile.userCredentials.secretManagerStoredSecurityToken'
      )

    if args.IsSpecified('salesforce_oauth2_client_id'):
      connection_profile.salesforceProfile.oauth2ClientCredentials.clientId = (
          args.salesforce_oauth2_client_id
      )
      update_fields.append('salesforceProfile.oauth2ClientCredentials.clientId')
    if args.IsSpecified('salesforce_oauth2_client_secret') or args.IsSpecified(
        'salesforce_secret_manager_stored_oauth2_client_secret'
    ):
      connection_profile.salesforceProfile.oauth2ClientCredentials.clientSecret = (
          args.salesforce_oauth2_client_secret
      )
      connection_profile.salesforceProfile.oauth2ClientCredentials.secretManagerStoredClientSecret = (
          args.salesforce_secret_manager_stored_oauth2_client_secret
      )
      update_fields.append(
          'salesforceProfile.oauth2ClientCredentials.clientSecret'
      )
      update_fields.append(
          'salesforceProfile.oauth2ClientCredentials.secretManagerStoredClientSecret'
      )

  def _UpdateSpannerProfile(self, connection_profile, args, update_fields):
    """Updates Spanner connection profile."""
    if args.IsSpecified('spanner_host'):
      connection_profile.spannerProfile.host = args.spanner_host
      update_fields.append('spannerProfile.host')

  def _UpdateMongodbProfile(self, connection_profile, args, update_fields):
    """Updates MongoDB connection profile."""
    if args.IsSpecified('mongodb_host_addresses'):
      addresses = []
      for host_address in args.mongodb_host_addresses:
        if args.mongodb_srv_connection_format:
          addresses.append(
              self._messages.HostAddress(hostname=host_address)
          )
        else:
          hostname, port = host_address.split(':')
          addresses.append(
              self._messages.HostAddress(hostname=hostname, port=int(port))
          )
      connection_profile.mongodbProfile.hostAddresses = addresses
      update_fields.append('monogodbProfile.hostAddresses')
    if args.IsSpecified('mongodb_replica_set'):
      connection_profile.mongodbProfile.replicaSet = args.mongodb_replica_set
      update_fields.append('mongodbProfile.replicaSet')
    if args.IsSpecified('mongodb_srv_connection_format') or args.IsSpecified(
        'mongodb_standard_connection_format'
    ):
      if args.mongodb_srv_connection_format:
        connection_profile.mongodbProfile.srvConnectionFormat = {}
      if args.mongodb_standard_connection_format:
        connection_profile.mongodbProfile.standardConnectionFormat = {}
      update_fields.append('mongodbProfile.srvConnectionFormat')
      update_fields.append('mongodbProfile.standardConnectionFormat')
    if args.IsSpecified('mongodb_username'):
      connection_profile.mongodbProfile.username = args.mongodb_username
      update_fields.append('mongodbProfile.username')
    if args.IsSpecified('mongodb_password') or args.IsSpecified(
        'mongodb_secret_manager_stored_password'
    ):
      connection_profile.mongodbProfile.password = args.mongodb_password
      connection_profile.mongodbProfile.secretManagerStoredPassword = (
          args.mongodb_secret_manager_stored_password
      )
      update_fields.append('mongodbProfile.password')
      update_fields.append('mongodbProfile.secretManagerStoredPassword')

  def _UpdateDataverseProfile(self, connection_profile, args, update_fields):
    """Updates Dataverse connection profile."""
    if not connection_profile.dataverseProfile:
      connection_profile.dataverseProfile = self._messages.DataverseProfile()

    if args.IsSpecified('dataverse_environment_url'):
      connection_profile.dataverseProfile.environmentUrl = (
          args.dataverse_environment_url
      )
      update_fields.append('dataverseProfile.environmentUrl')
    if args.IsSpecified('dataverse_tenant_id'):
      connection_profile.dataverseProfile.tenantId = args.dataverse_tenant_id
      update_fields.append('dataverseProfile.tenantId')

    # OAuth credentials
    if (
        args.IsSpecified('dataverse_oauth_client_id')
        or args.IsSpecified('dataverse_oauth_client_secret')
        or args.IsSpecified(
            'dataverse_secret_manager_stored_oauth_client_secret'
        )
    ):
      if not connection_profile.dataverseProfile.oauthClientCredentials:
        connection_profile.dataverseProfile.oauthClientCredentials = (
            self._messages.OauthClientCredentials()
        )

      if args.IsSpecified('dataverse_oauth_client_id'):
        connection_profile.dataverseProfile.oauthClientCredentials.clientId = (
            args.dataverse_oauth_client_id
        )
        update_fields.append('dataverseProfile.oauthClientCredentials.clientId')

      if args.IsSpecified('dataverse_oauth_client_secret') or args.IsSpecified(
          'dataverse_secret_manager_stored_oauth_client_secret'
      ):
        if (
            not connection_profile.dataverseProfile.oauthClientCredentials.clientSecret
        ):
          connection_profile.dataverseProfile.oauthClientCredentials.clientSecret = (
              self._messages.Secret()
          )

        if args.IsSpecified('dataverse_oauth_client_secret'):
          connection_profile.dataverseProfile.oauthClientCredentials.clientSecret.rawValue = (
              args.dataverse_oauth_client_secret
          )
          update_fields.append(
              'dataverseProfile.oauthClientCredentials.clientSecret.rawValue'
          )
        if args.IsSpecified(
            'dataverse_secret_manager_stored_oauth_client_secret'
        ):
          connection_profile.dataverseProfile.oauthClientCredentials.clientSecret.secretVersion = (
              args.dataverse_secret_manager_stored_oauth_client_secret
          )
          update_fields.append(
              'dataverseProfile.oauthClientCredentials.clientSecret.secretVersion'
          )

  def _UpdateSalesforceMarketingCloudProfile(
      self, connection_profile, args, update_fields
  ):
    """Updates Salesforce Marketing Cloud connection profile."""
    if not connection_profile.salesforceMarketingCloudProfile:
      connection_profile.salesforceMarketingCloudProfile = (
          self._messages.SalesforceMarketingCloudProfile()
      )

    if args.IsSpecified('salesforce_marketing_cloud_subdomain'):
      connection_profile.salesforceMarketingCloudProfile.subdomain = (
          args.salesforce_marketing_cloud_subdomain
      )
      update_fields.append('salesforceMarketingCloudProfile.subdomain')

    # OAuth credentials
    if (
        args.IsSpecified('salesforce_marketing_cloud_oauth_client_id')
        or args.IsSpecified('salesforce_marketing_cloud_oauth_client_secret')
        or args.IsSpecified(
            'salesforce_marketing_cloud_secret_manager_stored_oauth_client_secret'
        )
    ):
      if (
          not connection_profile.salesforceMarketingCloudProfile.oauthClientCredentials
      ):
        connection_profile.salesforceMarketingCloudProfile.oauthClientCredentials = (
            self._messages.OauthClientCredentials()
        )

      if args.IsSpecified('salesforce_marketing_cloud_oauth_client_id'):
        connection_profile.salesforceMarketingCloudProfile.oauthClientCredentials.clientId = (
            args.salesforce_marketing_cloud_oauth_client_id
        )
        update_fields.append(
            'salesforceMarketingCloudProfile.oauthClientCredentials.clientId'
        )

      if args.IsSpecified(
          'salesforce_marketing_cloud_oauth_client_secret'
      ) or args.IsSpecified(
          'salesforce_marketing_cloud_secret_manager_stored_oauth_client_secret'
      ):
        if (
            not connection_profile.salesforceMarketingCloudProfile.oauthClientCredentials.clientSecret
        ):
          connection_profile.salesforceMarketingCloudProfile.oauthClientCredentials.clientSecret = (
              self._messages.Secret()
          )

        if args.IsSpecified('salesforce_marketing_cloud_oauth_client_secret'):
          connection_profile.salesforceMarketingCloudProfile.oauthClientCredentials.clientSecret.rawValue = (
              args.salesforce_marketing_cloud_oauth_client_secret
          )
          update_fields.append(
              'salesforceMarketingCloudProfile.oauthClientCredentials.clientSecret.rawValue'
          )
        if args.IsSpecified(
            'salesforce_marketing_cloud_secret_manager_stored_oauth_client_secret'
        ):
          connection_profile.salesforceMarketingCloudProfile.oauthClientCredentials.clientSecret.secretVersion = (
              args.salesforce_marketing_cloud_secret_manager_stored_oauth_client_secret
          )
          update_fields.append(
              'salesforceMarketingCloudProfile.oauthClientCredentials.clientSecret.secretVersion'
          )

  def _UpdateServiceNowProfile(self, connection_profile, args, update_fields):
    """Updates ServiceNow connection profile."""
    if not connection_profile.serviceNowProfile:
      connection_profile.serviceNowProfile = self._messages.ServiceNowProfile()

    if args.IsSpecified('servicenow_instance'):
      connection_profile.serviceNowProfile.instance = args.servicenow_instance
      update_fields.append('serviceNowProfile.instance')

    # OAuth credentials
    if (
        args.IsSpecified('servicenow_oauth_client_id')
        or args.IsSpecified('servicenow_oauth_client_secret')
        or args.IsSpecified(
            'servicenow_secret_manager_stored_oauth_client_secret'
        )
    ):
      if not connection_profile.serviceNowProfile.oauthClientCredentials:
        connection_profile.serviceNowProfile.oauthClientCredentials = (
            self._messages.OauthClientCredentials()
        )

      if args.IsSpecified('servicenow_oauth_client_id'):
        connection_profile.serviceNowProfile.oauthClientCredentials.clientId = (
            args.servicenow_oauth_client_id
        )
        update_fields.append(
            'serviceNowProfile.oauthClientCredentials.clientId'
        )

      if args.IsSpecified('servicenow_oauth_client_secret') or args.IsSpecified(
          'servicenow_secret_manager_stored_oauth_client_secret'
      ):
        if (
            not connection_profile.serviceNowProfile.oauthClientCredentials.clientSecret
        ):
          connection_profile.serviceNowProfile.oauthClientCredentials.clientSecret = (
              self._messages.Secret()
          )

        if args.IsSpecified('servicenow_oauth_client_secret'):
          connection_profile.serviceNowProfile.oauthClientCredentials.clientSecret.rawValue = (
              args.servicenow_oauth_client_secret
          )
          update_fields.append(
              'serviceNowProfile.oauthClientCredentials.clientSecret.rawValue'
          )
        if args.IsSpecified(
            'servicenow_secret_manager_stored_oauth_client_secret'
        ):
          connection_profile.serviceNowProfile.oauthClientCredentials.clientSecret.secretVersion = (
              args.servicenow_secret_manager_stored_oauth_client_secret
          )
          update_fields.append(
              'serviceNowProfile.oauthClientCredentials.clientSecret.secretVersion'
          )

    # User-Password credentials
    if (
        args.IsSpecified('servicenow_username')
        or args.IsSpecified('servicenow_password')
        or args.IsSpecified('servicenow_secret_manager_stored_password')
    ):
      if not connection_profile.serviceNowProfile.userPasswordCredentials:
        connection_profile.serviceNowProfile.userPasswordCredentials = (
            self._messages.UserPasswordCredentials()
        )

      if args.IsSpecified('servicenow_username'):
        connection_profile.serviceNowProfile.userPasswordCredentials.username = (
            args.servicenow_username
        )
        update_fields.append(
            'serviceNowProfile.userPasswordCredentials.username'
        )

      if args.IsSpecified('servicenow_password') or args.IsSpecified(
          'servicenow_secret_manager_stored_password'
      ):
        if (
            not connection_profile.serviceNowProfile.userPasswordCredentials.password
        ):
          connection_profile.serviceNowProfile.userPasswordCredentials.password = (
              self._messages.Secret()
          )

        if args.IsSpecified('servicenow_password'):
          connection_profile.serviceNowProfile.userPasswordCredentials.password.rawValue = (
              args.servicenow_password
          )
          update_fields.append(
              'serviceNowProfile.userPasswordCredentials.password.rawValue'
          )
        if args.IsSpecified('servicenow_secret_manager_stored_password'):
          connection_profile.serviceNowProfile.userPasswordCredentials.password.secretVersion = (
              args.servicenow_secret_manager_stored_password
          )
          update_fields.append(
              'serviceNowProfile.userPasswordCredentials.password.secretVersion'
          )

  def _GetExistingConnectionProfile(self, name):
    get_req = (
        self._messages.DatastreamProjectsLocationsConnectionProfilesGetRequest(
            name=name
        )
    )
    return self._service.Get(get_req)

  def _UpdateLabels(self, connection_profile, args):
    """Updates labels of the connection profile."""
    add_labels = labels_util.GetUpdateLabelsDictFromArgs(args)
    remove_labels = labels_util.GetRemoveLabelsListFromArgs(args)
    value_type = self._messages.ConnectionProfile.LabelsValue
    update_result = labels_util.Diff(
        additions=add_labels,
        subtractions=remove_labels,
        clear=args.clear_labels
    ).Apply(value_type, connection_profile.labels)
    if update_result.needs_update:
      connection_profile.labels = update_result.labels

  def _GetUpdatedConnectionProfile(self, connection_profile, cp_type,
                                   release_track, args):
    """Returns updated connection profile and list of updated fields."""
    update_fields = []
    if args.IsSpecified('display_name'):
      connection_profile.displayName = args.display_name
      update_fields.append('displayName')

    if cp_type == 'MYSQL':
      self._UpdateMySqlProfile(
          connection_profile, args, update_fields)
    elif cp_type == 'ORACLE':
      self._UpdateOracleProfile(connection_profile, args, update_fields)
    elif cp_type == 'POSTGRESQL':
      self._UpdatePostgresqlProfile(connection_profile, args, update_fields)
    elif cp_type == 'SQLSERVER':
      self._UpdateSqlServerProfile(connection_profile, args, update_fields)
    elif cp_type == 'SALESFORCE':
      self._UpdateSalesforceProfile(connection_profile, args, update_fields)
    elif cp_type == 'DATAVERSE':
      self._UpdateDataverseProfile(connection_profile, args, update_fields)
    elif cp_type == 'SALESFORCE-MARKETING-CLOUD':
      self._UpdateSalesforceMarketingCloudProfile(
          connection_profile, args, update_fields
      )
    elif cp_type == 'SERVICENOW':
      self._UpdateServiceNowProfile(connection_profile, args, update_fields)
    elif cp_type == 'SPANNER':
      self._UpdateSpannerProfile(connection_profile, args, update_fields)
    elif cp_type == 'GOOGLE-CLOUD-STORAGE':
      self._UpdateGCSProfile(
          connection_profile, release_track, args, update_fields
      )
    elif cp_type == 'BIGQUERY':
      # There are currently no parameters that can be updated in a bigquery CP.
      pass
    elif cp_type == 'MONGODB':
      self._UpdateMongodbProfile(connection_profile, args, update_fields)
    else:
      raise exceptions.InvalidArgumentException(
          cp_type,
          'The connection profile type {0} is either unknown or not supported'
          ' yet.'.format(cp_type),
      )

    # TODO(b/207467120): deprecate BETA client.
    if release_track == base.ReleaseTrack.BETA:
      private_connectivity_ref = args.CONCEPTS.private_connection_name.Parse()
    else:
      private_connectivity_ref = args.CONCEPTS.private_connection.Parse()

    if private_connectivity_ref:
      connection_profile.privateConnectivity = (
          self._messages.PrivateConnectivity(
              privateConnectionName=private_connectivity_ref.RelativeName()
          )
      )
      update_fields.append('privateConnectivity')
    elif args.forward_ssh_hostname:
      self._UpdateForwardSshTunnelConnectivity(
          connection_profile, args, update_fields
      )
    elif args.static_ip_connectivity:
      connection_profile.staticServiceIpConnectivity = {}
      update_fields.append('staticServiceIpConnectivity')

    self._UpdateLabels(connection_profile, args)
    return connection_profile, update_fields

  def Create(self,
             parent_ref,
             connection_profile_id,
             cp_type,
             release_track,
             args=None):
    """Creates a connection profile.

    Args:
      parent_ref: a Resource reference to a parent datastream.projects.locations
        resource for this connection profile.
      connection_profile_id: str, the name of the resource to create.
      cp_type: str, the type of the connection profile ('MYSQL', ''
      release_track: Some arguments are added based on the command release
        track.
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      Operation: the operation for creating the connection profile.
    """
    self._ValidateArgs(args)

    connection_profile = self._GetConnectionProfile(cp_type,
                                                    connection_profile_id, args,
                                                    release_track)
    # TODO(b/207467120): only use flags from args.
    force = False
    if release_track == base.ReleaseTrack.BETA or args.force:
      force = True

    request_id = util.GenerateRequestId()
    create_req_type = (
        self._messages.DatastreamProjectsLocationsConnectionProfilesCreateRequest
    )
    create_req = create_req_type(
        connectionProfile=connection_profile,
        connectionProfileId=connection_profile.name,
        parent=parent_ref,
        requestId=request_id,
        force=force)

    return self._service.Create(create_req)

  def Update(self, name, cp_type, release_track, args=None):
    """Updates a connection profile.

    Args:
      name: str, the reference of the connection profile to
          update.
      cp_type: str, the type of the connection profile ('MYSQL', 'ORACLE')
      release_track: Some arguments are added based on the command release
        track.
      args: argparse.Namespace, The arguments that this command was
          invoked with.

    Returns:
      Operation: the operation for updating the connection profile.
    """
    self._ValidateArgs(args)

    current_cp = self._GetExistingConnectionProfile(name)

    updated_cp, update_fields = self._GetUpdatedConnectionProfile(
        current_cp, cp_type, release_track, args)

    # TODO(b/207467120): only use flags from args.
    force = False
    if release_track == base.ReleaseTrack.BETA or args.force:
      force = True

    request_id = util.GenerateRequestId()
    update_req_type = (
        self._messages.DatastreamProjectsLocationsConnectionProfilesPatchRequest
    )
    update_req = update_req_type(
        connectionProfile=updated_cp,
        name=updated_cp.name,
        updateMask=','.join(update_fields),
        requestId=request_id,
        force=force,
    )

    return self._service.Patch(update_req)

  def List(self, project_id, args):
    """Get the list of connection profiles in a project.

    Args:
      project_id: The project ID to retrieve
      args: parsed command line arguments

    Returns:
      An iterator over all the matching connection profiles.
    """
    location_ref = self._resource_parser.Create(
        'datastream.projects.locations',
        projectsId=project_id,
        locationsId=args.location,
    )

    list_req_type = (
        self._messages.DatastreamProjectsLocationsConnectionProfilesListRequest
    )
    list_req = list_req_type(
        parent=location_ref.RelativeName(),
        filter=args.filter,
        orderBy=','.join(args.sort_by) if args.sort_by else None,
    )

    return list_pager.YieldFromList(
        service=self._client.projects_locations_connectionProfiles,
        request=list_req,
        limit=args.limit,
        batch_size=args.page_size,
        field='connectionProfiles',
        batch_size_attribute='pageSize')

  def Discover(self, parent_ref, release_track, args):
    """Discover a connection profile.

    Args:
      parent_ref: a Resource reference to a parent datastream.projects.locations
        resource for this connection profile.
      release_track: Some arguments are added based on the command release
        track.
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      Operation: the operation for discovering the connection profile.
    """
    request = self._messages.DiscoverConnectionProfileRequest()
    if args.connection_profile_name:
      connection_profile_ref = args.CONCEPTS.connection_profile_name.Parse()
      request.connectionProfileName = connection_profile_ref.RelativeName()
    elif args.connection_profile_object_file:
      request.connectionProfile = self._ParseConnectionProfileObjectFile(
          args.connection_profile_object_file, release_track
      )

    if args.recursive or args.full_hierarchy:
      request.fullHierarchy = True
    elif args.recursive_depth:
      request.hierarchyDepth = (int)(args.recursive_depth)
    elif args.hierarchy_depth:
      request.hierarchyDepth = (int)(args.hierarchy_depth)
    else:
      request.fullHierarchy = False

    if args.mysql_rdbms_file:
      request.mysqlRdbms = util.ParseMysqlRdbmsFile(self._messages,
                                                    args.mysql_rdbms_file,
                                                    release_track)
    elif args.oracle_rdbms_file:
      request.oracleRdbms = util.ParseOracleRdbmsFile(self._messages,
                                                      args.oracle_rdbms_file,
                                                      release_track)
    elif args.postgresql_rdbms_file:
      request.postgresqlRdbms = util.ParsePostgresqlRdbmsFile(
          self._messages, args.postgresql_rdbms_file)
    elif args.sqlserver_rdbms_file:
      request.sqlServerRdbms = util.ParseSqlServerRdbmsFile(
          self._messages, args.sqlserver_rdbms_file
      )
    discover_req_type = (
        self._messages.DatastreamProjectsLocationsConnectionProfilesDiscoverRequest
    )
    discover_req = discover_req_type(
        discoverConnectionProfileRequest=request, parent=parent_ref)
    return self._service.Discover(discover_req)

  def GetUri(self, name):
    """Get the URL string for a connection profile.

    Args:
      name: connection profile's full name.

    Returns:
      URL of the connection profile resource
    """

    uri = self._resource_parser.ParseRelativeName(
        name, collection='datastream.projects.locations.connectionProfiles')
    return uri.SelfLink()
